from ldappy.connection.ldapclient import Ldapclient
import json
import logging
from typing import Any, List, Optional, Union, Dict

import ldap3

logger = logging.getLogger(__name__)


class Ldap(Ldapclient):
    """ Class for LDAP related connections/operations. """

    @staticmethod
    def get_value(attribute: ldap3.abstract.attribute.Attribute) -> Any:
        """ Retourne la valeur d'un attribut ldap spécifique

        Args:
            attribute (ldap3.abstract.attribute.Attribute): un attribut ldap de type ldap3

        Returns:

            Valeur de l'attribut avec un typage normal (str, int, List, ... )

        """
        logger.debug(f'Type de la valeur recherchée : {attribute.value}')
        return attribute.value

    @staticmethod
    def get_entry_json(entry: ldap3.Connection.entries) -> Dict:
        """ Transforme les entrées ldap3 en dictionnaire

        Args:
            entry (ldap3.Connection.entries): La liste des informations qui correspond à la résultat d'une recherche

        Returns:
            Un dictionnaire des informations d'un "entry" ldap3
        """
        data = entry.entry_to_json()
        return json.loads(data)

    def who_am_i(self) -> str:
        """ Connaitre le status d'authentification sur le sereur.

        Returns:
            Le status d'authentification sous forme de chaine de caractère.
        """
        return self.conn.extend.standard.who_am_i()

    def _generic_search(self, search_filter: str, *,
                        branch: str,
                        attributes: Optional[Union[List[str], None]] = ldap3.ALL_ATTRIBUTES) -> ldap3.Connection.entries:
        """ Récupère les informations d'un individu (dans notre cas 'ou=People') par rapport à un élément de recherche

        Args:
            search_filter(str) : Filtre de recherche ldap
            branch (str): La branche sur laquelle on se place
            attributes (List[str]):  Optionnel. Les attributs ldap qui seront retournées. Par défaut,
                            tous les attributs sont retournés. Mais on peut spécifier une liste d'attribut ldap

        Returns:
            La liste des entrées ldap par rapport à une valeur d'un attribut donné.
        """
        self.conn.search(branch, f'({search_filter})',
                         search_scope=ldap3.SUBTREE, attributes=attributes)
        return self.conn.entries

    def _generic_modify_add(self, attribute: str, new_value: List[str], *, dn: str) -> Dict[str, Union[int, str, bool]]:
        """ Ajoute une valeur à un attribut passé en paramètre.

        Args:
            attribute(str) : L'attribut ldap où on va ajouter une valeur
            new_value(str): La valeur à ajouter
            dn (str): Le dn sur laquelle on se place

        Returns:
            Un dictionnaire décrivant le succès ou non de l'opération.
        """
        try:
            self.conn.modify(dn, {attribute: [(ldap3.MODIFY_ADD, new_value)]})
            if self.conn.result['description'] == 'success':
                return {'code': 10,
                        'msg': 'Ajout effectué',
                        'debug': f"{dn} : {new_value} a été ajouté avec succès à l'attribut {attribute}",
                        'success': True}
            elif self.conn.result['description'] == 'attributeOrValueExists':
                return {'code': 20,
                        'msg': 'Valeur existe déjà',
                        'debug': f"{dn} : {new_value} existe déjà dans l'attribut {attribute}",
                        'success': False}
            else:
                return {'code': 25,
                        'msg': 'Un problème est survenue',
                        'debug': f"{dn} : {new_value} n'a pas pu être ajouté à l'attribut {attribute}",
                        'success': False}
        except ldap3.core.exceptions.LDAPInvalidDnError as invalid_dn:
            return {'code': 99,
                    'msg': str(invalid_dn),
                    'debug': f"Le dn {dn} n'est pas valide",
                    'success': False}

    def _generic_modify_delete(self, attribute: str, new_value: List[str], *, dn: str) -> Dict[str, Union[str, bool]]:
        """ Supprime une valeur à un attribut passé en paramètre.

        Args:
            attribute(str) : L'attribut ldap où on va supprimer une valeur
            new_value(str): La valeur à supprimer
            dn (str): Le dn sur laquelle on se place

        Returns:
            Un dictionnaire décrivant le succès ou non de l'opération.
        """
        try:
            self.conn.modify(dn, {attribute: [(ldap3.MODIFY_DELETE, new_value)]})
            logger.debug(self.conn.result)
            if self.conn.result['description'] == 'success':
                return {'code': 12,
                        'msg': 'Suppression effectueé',
                        'debug': f"{dn} : {new_value} a été supprimé avec succès à l'attribut {attribute}",
                        'success': True}
            elif self.conn.result['description'] == 'noSuchAttribute':
                return {'code': 22,
                        'msg': "Valeur n'existe pas",
                        'debug': f"{dn} : {new_value} n'existe pas dans l'attribut {attribute}",
                        'success': False}
            else:
                return {'code': 25,
                        'msg': 'Un problème est survenue',
                        'debug': f"{dn} : {new_value} n'a pas pu être ajouté à l'attribut {attribute}",
                        'success': False}
        except ldap3.core.exceptions.LDAPInvalidDnError as invalid_dn:
            return {'code': 99,
                    'msg': str(invalid_dn),
                    'debug': f"Le dn {dn} n'est pas valide",
                    'success': False}
